import 'package:dynatrace_flutter_plugin/src/agent/agent_impl.dart';
import 'package:dynatrace_flutter_plugin/src/agent/model/configuration.dart';
import 'package:dynatrace_flutter_plugin/src/agent/model/log_level.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';

import '../util/mock_method_channel.dart';
import '../widget.dart';

void main() {
  MockMethodChannel methodChannel = MockMethodChannel();
  late DynatraceImpl dynatrace;

  setUp(() {
    Map<String, bool> configuration = {};
    configuration["autoStart"] = false;

    when(methodChannel.invokeMapMethod("getAutoStartConfiguration"))
        .thenAnswer((_) => Future<Map<String, bool>>.value(configuration));

    dynatrace =
        DynatraceImpl.private(methodChannel, startUpFunction: (widget) => {});
  });

  test('Crash Reporting On', () async {
    // Default Startup Method
    await dynatrace.start(const TestMainWidget(),
        configuration: const Configuration());

    dynatrace.reportCrash("errorName", "reason", "stacktrace");

    verify(methodChannel.invokeMethod("reportCrash", any)).called(1);

    // Removed OFF Test as the agent now decides about sending
  });

  test('Crash Reporting On with startWithoutWidget()', () async {
    // Default Startup Method
    await dynatrace.startWithoutWidget(configuration: const Configuration());
    runApp(const TestMainWidget());

    dynatrace.reportCrash("errorName", "reason", "stacktrace");

    verify(methodChannel.invokeMethod("reportCrash", any)).called(1);

    // Removed OFF Test as the agent now decides about sending
  });

  test('Action not created before start', () {
    dynatrace.enterAction("Hello");

    verifyNever(methodChannel.invokeMethod("enterAction", any));
  });

  test('AutoStartProperty', () {
    Configuration example = const Configuration();
    expect(example.isAutoStartPropertyAvailable, false);

    example = const Configuration(applicationId: "");
    expect(example.isAutoStartPropertyAvailable, false);

    example = const Configuration(beaconUrl: "");
    expect(example.isAutoStartPropertyAvailable, false);

    example = const Configuration(applicationId: "", beaconUrl: "");
    expect(example.isAutoStartPropertyAvailable, false);

    example = const Configuration(applicationId: "", beaconUrl: null);
    expect(example.isAutoStartPropertyAvailable, false);

    example = const Configuration(applicationId: "X", beaconUrl: "X");
    expect(example.isAutoStartPropertyAvailable, true);
  });

  test('Manual Startup Properties', () {
    Configuration example = const Configuration();
    expect(example.getStartupConfiguration().isEmpty, true);

    example = const Configuration(applicationId: "id");
    expect(
        example.getStartupConfiguration().containsKey("applicationId"), true);

    example = const Configuration(beaconUrl: "id");
    expect(example.getStartupConfiguration().containsKey("beaconUrl"), true);

    example = const Configuration(reportCrash: false);
    expect(
        example.getStartupConfiguration().containsKey("crashReporting"), true);

    example = const Configuration(userOptIn: true);
    expect(example.getStartupConfiguration().containsKey("userOptIn"), true);

    example = const Configuration(certificateValidation: false);
    expect(
        example.getStartupConfiguration().containsKey("certificateValidation"),
        true);

    example = const Configuration(logLevel: LogLevel.Debug);
    expect(example.getStartupConfiguration().containsKey("logLevel"), true);
  });
}
