import 'package:dynatrace_flutter_plugin/src/agent/action/counter.dart';
import 'package:dynatrace_flutter_plugin/src/agent/agent_impl.dart';
import 'package:dynatrace_flutter_plugin/src/agent/navigation_observer.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';

import '../util/mock_method_channel.dart';
import '../widget.dart';

class MockRoute extends Mock implements PageRoute {
  @override
  RouteSettings settings;

  MockRoute({required this.settings});
}

void main() {
  MockMethodChannel methodChannel = MockMethodChannel();
  DynatraceImpl dynatrace;
  late DynatraceNavigationObserver observer;

  setUp(() async {
    Map<String, bool> configuration = {};
    configuration["autoStart"] = true;
    configuration["crashReporting"] = true;
    configuration["webRequest"] = true;
    configuration["logLevel"] = true;

    when(methodChannel.invokeMapMethod("getAutoStartConfiguration"))
        .thenAnswer((_) => Future<Map<String, bool>>.value(configuration));

    dynatrace = DynatraceImpl.private(methodChannel);
    await dynatrace.start(const TestMainWidget());
    observer = DynatraceNavigationObserver.private(dynatrace: dynatrace);
    ActionCounter().resetCount();
  });

  test('Navigation Change Push', () {
    const RouteSettings(name: "NewRoute");
    observer.didPush(MockRoute(settings: const RouteSettings(name: "NewRoute")),
        MockRoute(settings: const RouteSettings(name: "PreviousRoute")));
    observer.didPush(MockRoute(settings: const RouteSettings(name: null)),
        MockRoute(settings: const RouteSettings(name: null)));

    verify(methodChannel.invokeMethod("enterAction", {
      "name": "Navigated to screen NewRoute",
      "key": 0,
      "platform": null
    })).called(1);
    verify(methodChannel.invokeMethod("leaveAction", {"key": 0})).called(1);
  });

  test('Navigation Change Pop', () {
    const RouteSettings(name: "NewRoute");
    observer.didPop(MockRoute(settings: const RouteSettings(name: "NewRoute")),
        MockRoute(settings: const RouteSettings(name: "PreviousRoute")));
    observer.didPop(MockRoute(settings: const RouteSettings(name: null)),
        MockRoute(settings: const RouteSettings(name: null)));

    verify(methodChannel.invokeMethod("enterAction", {
      "name": "Navigated to screen PreviousRoute",
      "key": 0,
      "platform": null
    })).called(1);
    verify(methodChannel.invokeMethod("leaveAction", {"key": 0})).called(1);
  });

  test('Navigation Change Replace', () {
    const RouteSettings(name: "NewRoute");
    observer.didReplace(
        newRoute: MockRoute(settings: const RouteSettings(name: "NewRoute")),
        oldRoute:
            MockRoute(settings: const RouteSettings(name: "PreviousRoute")));
    observer.didReplace(
        newRoute: MockRoute(settings: const RouteSettings(name: null)),
        oldRoute: MockRoute(settings: const RouteSettings(name: null)));

    verify(methodChannel.invokeMethod("enterAction", {
      "name": "Navigated to screen NewRoute",
      "key": 0,
      "platform": null
    })).called(1);
    verify(methodChannel.invokeMethod("leaveAction", {"key": 0})).called(1);
  });
}
