import 'package:dynatrace_flutter_plugin/dynatrace_flutter_plugin.dart';
import 'package:dynatrace_flutter_plugin/src/agent/agent_impl.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';

import '../util/mock_method_channel.dart';
import '../widget.dart';

void main() {
  late MockMethodChannel methodChannel;
  late Dynatrace dynatrace;

  setUp(() async {
    methodChannel = MockMethodChannel();
    dynatrace = DynatraceImpl.private(methodChannel);
  });

  group('User Privacy Options', () {
    test('Apply Privacy Options when Agent is disabled', () {
      dynatrace.applyUserPrivacyOptions(
          UserPrivacyOptions(DataCollectionLevel.UserBehavior, true));

      // Verify if Method channel was called
      verifyNever(methodChannel.invokeMethod("applyUserPrivacyOptions", any));
    });

    test('Apply Privacy Options when Agent is enabled', () async {
      Map<String, bool> configuration = {};
      configuration["autoStart"] = true;
      configuration["crashReporting"] = true;
      configuration["webRequest"] = true;
      configuration["logLevel"] = true;

      when(methodChannel.invokeMapMethod("getAutoStartConfiguration"))
          .thenAnswer((_) => Future<Map<String, bool>>.value(configuration));

      await dynatrace.start(const TestMainWidget(),
          configuration: const Configuration());

      dynatrace.applyUserPrivacyOptions(
          UserPrivacyOptions(DataCollectionLevel.UserBehavior, true));

      verify(methodChannel.invokeMethod("applyUserPrivacyOptions", {
        "crashReportingOptedIn": true,
        "dataCollectionLevel": 2,
        "platform": null
      })).called(1);
    });

    test('Get Privacy Options when Agent is disabled', () async {
      UserPrivacyOptions options = await dynatrace.getUserPrivacyOptions();
      expect(options.crashReportingOptedIn, false);
      expect(options.dataCollectionLevel, DataCollectionLevel.Off);
    });

    test('Get Privacy Options when Agent is enabled', () async {
      Map<String, bool> configuration = {};
      configuration["autoStart"] = true;
      configuration["crashReporting"] = true;
      configuration["webRequest"] = true;
      configuration["logLevel"] = true;

      when(methodChannel.invokeMapMethod("getAutoStartConfiguration"))
          .thenAnswer((_) => Future<Map<String, bool>>.value(configuration));

      await dynatrace.start(const TestMainWidget(),
          configuration: const Configuration());

      Map<String, dynamic> userPrivacyOptions = {};
      userPrivacyOptions["dataCollectionLevel"] = 2;
      userPrivacyOptions["crashReportingOptedIn"] = true;

      when(methodChannel.invokeMapMethod<String, dynamic>(
              "getUserPrivacyOptions", any))
          .thenAnswer(
              (_) => Future<Map<String, dynamic>>.value(userPrivacyOptions));

      UserPrivacyOptions options = await dynatrace.getUserPrivacyOptions();
      expect(options.crashReportingOptedIn, true);
      expect(options.dataCollectionLevel, DataCollectionLevel.UserBehavior);
    });
  });
}
