import 'package:dynatrace_flutter_plugin/src/cli/config/model/cli_arguments.dart';
import 'package:dynatrace_flutter_plugin/src/cli/util/pathConstants.dart';

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';

class MockPaths extends Mock implements PathConstants {
  @override
  String getConfigurationPath() {
    return super.noSuchMethod(Invocation.method(#getConfigurationPath, []),
        returnValue: "", returnValueForMissingStub: "");
  }

  @override
  String getAndroidGradle() {
    return super.noSuchMethod(Invocation.method(#getAndroidGradle, []),
        returnValue: "", returnValueForMissingStub: "");
  }

  @override
  String getIosPListFile() {
    return super.noSuchMethod(Invocation.method(#getIosPListFile, []),
        returnValue: "", returnValueForMissingStub: "");
  }
}

void main() {
  test('Default Values', () async {
    MockPaths paths = MockPaths();

    when(paths.getConfigurationPath()).thenReturn("CONFIG_PATH");
    when(paths.getAndroidGradle()).thenReturn("ANDROID_GRADLE");
    when(paths.getIosPListFile()).thenReturn("IOS_PLIST");

    CommandLineArguments arguments =
        CommandLineArguments.parseArgumentList(paths, []);

    expect(arguments.configPath, "CONFIG_PATH");
    expect(arguments.gradlePath, "ANDROID_GRADLE");
    expect(arguments.plistPath, "IOS_PLIST");
    expect(arguments.isDebug, false);
    expect(arguments.isUninstall, false);
  });

  test('Set Values', () async {
    MockPaths paths = MockPaths();

    CommandLineArguments arguments = CommandLineArguments.parseArgumentList(
        paths, [
      "--uninstall",
      "--debug",
      "--config=SET1",
      "--gradle=SET2",
      "--plist=SET3"
    ]);

    expect(arguments.configPath, "SET1");
    expect(arguments.gradlePath, "SET2");
    expect(arguments.plistPath, "SET3");
    expect(arguments.isDebug, true);
    expect(arguments.isUninstall, true);
  });
}
